grammar CSimple;

options {
	language = Java;
	output = AST;
	backtrack = true;
}

tokens {
    TYPEDEF		= 'typedef';
    STRUCT		= 'struct';
    STATIC		= 'static';
    INLINE    = 'inline';
    EXTERN		= 'extern';
    CONST			= 'const';
    ATTR      = '__attribute__';
    HEXPREFIX	= '0x';
}

@header {
  package de.sm.csimple;
  
  import java.io.File;
}

@lexer::header {
  package de.sm.csimple;
}

@lexer::members {
  private int lineCount = 0;
  
  public int getLineCount() {
    return lineCount;
  }
}

@members {
	private List<String> functions = new ArrayList<String>();
	
	public List<String> getFuntions() {
		return functions;
	}
	
	private File parsedFile;
  
  /**
  * Make like error reporting for parser errors allows to benefit
  * from IDE make integration.
  */
  @Override
  public void reportError(RecognitionException re) {
    String errorMsg = "";
    if (parsedFile != null) {
      errorMsg += parsedFile.getAbsolutePath();
    }
    
    errorMsg += ":" + re.line + ": warning: Non critical parse error, file is ignored: ";
    System.err.print(errorMsg);
    super.reportError(re);
  }
  
  /**
  * Set the file which is actually parsed.
  * Is used to be able to generate meaningful error messages.
  * Is not used for parsing at all.
  */
  public void setParsedFile(File file) {
    parsedFile = file;
  }
}


definitions:
	(typedefinition | struct | function | preprocessor | variable | NEWLINE | externDef )*
	EOF
;

externDef:
  EXTERN QUOTE IDENTIFIER QUOTE 
  LBRACE
  (typedefinition | struct | function | preprocessor | variable | NEWLINE )*
  RBRACE
;

literalValue:
	IDENTIFIER | DIGIT+ | hexValue | content
;

variable:
	(STATIC | EXTERN)?
	typeRef  
	IDENTIFIER 
	('[' (literalValue)? ']')?
	( '=' literalValue )?
	';'
;

preprocessor :
  '#'
  ((~( PPCONTINUE | NEWLINE ))* PPCONTINUE)*
  (~( PPCONTINUE | NEWLINE ))*
  //(options {greedy=false;} : (~(LBRACE | RBRACE | ';' | NEWLINE) )+ )
  NEWLINE		// may be this is not the best way, but will work for now
;

typedefinition:
	TYPEDEF	def
;

struct:
	STRUCT def
	| (IDENTIFIER '(' IDENTIFIER ')' content ';') // its some CSOM sepecific macro
;

def:
	(content | ~(';' | LBRACE))*
	';'
;

function:
	(STATIC | EXTERN)? (INLINE)?
	returnType=typeRef name=typeRef '(' ~(')')* ')'
	(content 
	 | (ATTR '(' '(' IDENTIFIER ')' ')')?
	   ';')
	
	{	functions.add($name.text.intern());	}
;

contentcontent:
	(content | (~(LBRACE | RBRACE))+)*;

content :
  LBRACE contentcontent RBRACE
;

typeRef:
	CONST?
	(IDENTIFIER
	 | IDENTIFIER '(' IDENTIFIER ')'  // some macros used in CSOM
	) ('*')*
;

hexValue:
	HEXPREFIX (DIGIT | 'a'..'f' | 'A'..'F')+
;

SPECIAL : '(' | ')' | ':' | '='| '.' | '*' | '+' | '%' | '#' | '/' | '\\' | '^' | '?' | '@' | '$'
	| '_' | ';' | ',' | '|' | '&' | '-' | '<' | '>' | '[' | ']' | '!' | '\'' | '~' ;

IDENTIFIER : (LETTER | '_') ( DIGIT | LETTER | '_' )*;

DIGIT
    :    '0'..'9'
    ;
fragment LETTER
    : 'a'..'z'
    | 'A'..'Z'
    ;

PPCONTINUE: '\\' NEWLINE;

NEWLINE
  : '\r'? '\n'
  { lineCount++; }
  //{ channel=HIDDEN; }
;

WS
  : (' '|'\t')+		//|'\n'|'\r' 
  { channel=HIDDEN; }
;

COMMENT
  : '//' (~('\n'|'\r'))* NEWLINE
  { channel=HIDDEN; }
;
  
// multiple-line comments
ML_COMMENT
  : '/*' (options {greedy=false;} : .)* '*/' 
  { channel=HIDDEN; }
;

QUOTE  : '"';
LBRACE : '{';
RBRACE : '}';