package de.sm.metrics;

import java.io.File;
import java.io.IOException;

import org.antlr.runtime.ANTLRFileStream;
import org.antlr.runtime.CharStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.RecognitionException;

import de.sm.csimple.CSimpleLexer;
import de.sm.csimple.CSimpleParser;
import de.sm.vmadl.CombinationRegistry;
import de.sm.vmadl.ServiceRegistry;
import de.sm.vmadl.VmadlLexer;
import de.sm.vmadl.VmadlParser;
import de.sm.vmadl.VmadlParser.architecture_return;
import de.sm.vmadl.elements.ServiceModule;

public class CMetrics extends Metrics {

	private CSimpleParser parser;
	
	public CMetrics(File file) throws IOException, RecognitionException {
		super(file);
		parseFile();
	}
	
	private void parseFile() throws IOException, RecognitionException {
		CharStream input = new ANTLRFileStream(file.getAbsolutePath());
		CSimpleLexer lexer = new CSimpleLexer(input);
        CommonTokenStream tokens = new  CommonTokenStream(lexer);
        parser = new CSimpleParser(tokens);
		parser.setParsedFile(file);
	    parser.definitions();
	}
	
	public int getFunctionCount() {
		return parser.getFunctionCount();
	}
	
	public int getFunctionSigCount() {
		return parser.getFunctionSigCount();
	}
	
	public int getPreprocessorCount() {
		return parser.getPreprocessorCount();
	}
	
	public int getStructCount() {
		return parser.getStructCount();
	}
	
	public int getTypedefCount() {
		return parser.getTypedefCount();
	}
	
	public int getVariableCount() {
		return parser.getVariableCount();
	}
}
