package de.sm.metrics;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.antlr.runtime.RecognitionException;

/**
 * Collect the metrics.
 * @author Stefan Marr
 */
public class Collector {
	
	private ModuleRegistry reg;
	private FileSelector fileSelector;

	public Collector(ModuleRegistry reg, FileSelector fileSelector) {
		this.reg = reg;
		this.fileSelector = fileSelector;
	}

	/**
	 * Collect the metrics.
	 */
	public void collect() {
		collectLineCounts();
		collectCStats();
		collectAspectCStats();
		collectVmadlStats();
	}

	private void collectCStats() {
		List<File> files = fileSelector.getFilesForCStats();
		
		for (File file : files) {
			try {
				CMetrics c = new CMetrics(file);
				ImplementationModule module =
					(ImplementationModule) reg.get(file);
				module.setFunctionCount(c.getFunctionCount());
				module.setFunctionSigCount(c.getFunctionSigCount());
				module.setPreprocessorCount(c.getPreprocessorCount());
				module.setStructCount(c.getStructCount());
				module.setTypedefCount(c.getTypedefCount());
				module.setVariableCount(c.getVariableCount());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (RecognitionException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	private void collectAspectCStats() {
		List<File> files = fileSelector.getFilesForAspectCStats();
		
		for (File file : files) {
			try {
				AspectCMetrics c = new AspectCMetrics(file);
				ImplementationModule module =
					(ImplementationModule) reg.get(file);
				module.setAdviceCount(c.getAdviceCount());
				module.setPointcutCount(c.getPointcutCount());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (RecognitionException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	private void collectVmadlStats() {
		List<File> files = fileSelector.getFilesForVmadlStats();
		
		for (File file : files) {
			try {
				VmadlMetrics c = new VmadlMetrics(file);
				ImplementationModule module =
					(ImplementationModule) reg.get(file);
				module.setAdviceCount(c.getAdviceCount());
				module.setPointcutCount(c.getPointcutCount());
				module.setCombineCount(c.getCombineCount());
				module.setDeclarationCount(c.getDeclarationCount());
				module.setFunctionCount(c.getFunctionCount());
				module.setPreprocessorCount(c.getPreprocessorCount());
				module.setTypeCount(c.getTypeCount());
				module.setVariableCount(c.getVariableCount());
				
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (RecognitionException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private void collectLineCounts() {
		List<File> files = fileSelector.getFilesForLineCount();
		
		for (File file : files) {
			try {
				LinesOfCode loc = new LinesOfCode(file);
				ImplementationModule module =
					(ImplementationModule) reg.get(file);
				module.setLinesOfCode(loc.getLinesInFile());
				module.setLinesOfCodeWithStatements(
						loc.getLineCountWithStatements());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

}
