/**
 * 
 */
package de.sm.metrics;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.Arrays;

import org.antlr.runtime.ANTLRFileStream;
import org.antlr.runtime.CharStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.Token;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;

import de.sm.csimple.CSimpleLexer;

/**
 * @author gron
 *
 */
public class Main {

	/**
	 * CLI options.
	 */
	private Options options;
	
	/**
	 * Holds the files which need to be analyzed.
	 */
	private FileSelector fileSelector;

	private ModuleRegistry reg;

	private Collector collector;

	/**
	 * Initialize the command-line parameter options.
	 */
	private void createOptions() {
		options = new Options();
//		options.addOption("test", false, "if the test option is set,"
//				+ " output to test generator is produced");
//		options.addOption("f", "file", true, "output will be generated"
//				+ " into files with given prefix if this option isn't used,"
//				+ " output will be splited up into separate files for"
//				+ " each service.");
//		options.addOption("o", "output-path", true, 
//				"output will be generated into given folder");
		options.addOption("?", "help", false, "prints this help message");
	}	
	
	/**
	 * Main method for this application instance.
	 * @param args parameters from the CLI
	 */
	public final void run(final String[] args) {
		createOptions();
		
		// create the command line parser
		CommandLineParser parser = new PosixParser();
		CommandLine line;
		try {
			line = parser.parse(options, args);
			
			String[] arguments = line.getArgs();
			
			if (arguments.length < 1 || line.hasOption('?')) {
				printUsage();
				return;
			}
			
			reg = new ModuleRegistry(new File(arguments[0]));
			
			fileSelector = new FileSelector(arguments[0]);
			fileSelector.collect();
			
			collector = new Collector(reg, fileSelector);
			collector.collect();
			
			Presenter p = new Presenter(reg);
			p.showMetrics();
			
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			printUsage();
		}
	}
	
	/**
	 * Output command-line help to the user.
	 */
	private void printUsage() {
		HelpFormatter formatter = new HelpFormatter();
		formatter.printHelp("metrics <project-folder>",
				"metrics will parse the given project folder and all "
				+ "relevant files to compute metrics for this project."
				, options, "", true);
		
	}
	
	/**
	 * @param args CLI parameter given to this application
	 */
	public static void main(final String[] args) {
		Main app = new Main();
		app.run(args);
	}
	
	

}
