package de.sm.metrics;

import java.io.File;
import java.security.InvalidParameterException;
import java.util.HashMap;
import java.util.Map;

/**
 * Modules are ambigious units of granularity.
 * On the one hand, they are the conceptual entities
 * representing a set of implementation entities, but on the other hand
 * they are also implementation modules i.e. files.
 * 
 * A conceptual module MUST NOT be used to have a real file as its origin.
 * Instead this file is a sub module.
 * 
 * @author Stefan Marr
 *
 */
abstract public class Module {
	
	private File file;
	private ConceptualModule parent;
	
	public Module(File file) {
		this.file = file;
	}
	
	abstract public int getLinesOfCode();
	
	abstract public int getLinesOfCodeWithStatements();
	
	public File getFile() {
		return file;
	}

	public void setParent(ConceptualModule module) {
		parent = module;
	}
	
	public String getName() {
		return file.getName();
	}

	abstract public int getSubModuleCount();
	
}
