package de.sm.odl.elements;

import java.util.List;

import de.sm.vmadl.elements.ServiceModule;

public class TempType extends Type {
	private Type initializedType = null;
	
	public void initialize(Type type) {
		if (initializedType != null) throw new IllegalStateException("Type has already been initialized.");
		
		// Copy definitions from temporary placeholder to the real type instance
		type.setBaseclass(getBaseclass());
		type.getFields().addAll(getFields());
		type.getFunctions().addAll(getFunctions());
		type.getTraits().addAll(getTraits());
		
		// now this instance will be used as a proxy, 
		// because we can't change all references to use the right object
		initializedType = type;
	}
	
	public void setModule(ServiceModule module) {
		throw new IllegalStateException("Set module can not be called on a temporary module. Go to the real object instead.");
	}
	
	public ServiceModule getModule() {
		if (initializedType != null) {
			return initializedType.getModule();
		} else {
			return super.getModule();
		}
	}
	
	public Type getRealType() {
		return initializedType;
	}
	
	public void setName(String name) {
		if (initializedType != null) {
			initializedType.setName(name);
		} else {
			super.setName(name);
		}
	}
	
	public String getName() {
		if (initializedType != null) {
			return initializedType.getName();
		} else {
			return super.getName();
		}
	}
	
	public void setBaseclass(Type baseclass) {
		if (initializedType != null) {
			initializedType.setBaseclass(baseclass);
		} else {
			super.setBaseclass(baseclass);
		}
	}
	
	public Type getBaseclass() {
		if (initializedType != null) {
			return initializedType.getBaseclass();
		} else {
			return super.getBaseclass();
		}
	}
	
	public void addTrait(TraitType type) {
		if (initializedType != null) {
			initializedType.addTrait(type);
		} else {
			super.addTrait(type);
		}
	}
	
	public List<TraitType> getTraits() {
		if (initializedType != null) {
			return initializedType.getTraits();
		} else {
			return super.getTraits();
		}
	}
}
