/**
 * 
 */
package de.sm.vmadl;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Arrays;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;

/**
 * @author Stefan Marr
 *
 */
public class Main {

	//private boolean testFlag;
	private Options options;
	private String[] input;
	private String outputDir;
	private Vmadl vmadl;

	//private String getCurrentWorkingDirectory()  {
	//	return System.getProperty("user.dir");
	//}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			Main app = new Main();
			app.run(args);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}
	}
	
	public void run(String[] args) {
		createOptions();
		
		// create the command line parser
		CommandLineParser parser = new PosixParser();
		CommandLine line;
		try {
			line = parser.parse(options, args);
			
			//testFlag = line.hasOption("test");
			//String outputPath = line.getOptionValue('o');
			//String outputFilename = line.getOptionValue('f');
			
			String[] arguments = line.getArgs();
			
			if (arguments.length < 2 || line.hasOption('?')) {
				printUsage();
				return;
			}
			
			input = Arrays.copyOfRange(arguments, 0, arguments.length-1);
			outputDir = arguments[arguments.length - 1];
			
			loadVmadl();
			generateImplementation();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			printUsage();
		}
	}
	
	private void loadVmadl() {
		vmadl = new Vmadl();
//		File dir = new File(inputDir);
//		FilenameFilter filter = new FilenameFilter() { public boolean accept(File directory, String filename) { return filename.endsWith(".vmadl"); }};
//		File[] archFiles = dir.listFiles(filter);
		
		//for (File archDesc : archFiles) {
		for (String archDesc : input) {
			vmadl.loadFromFile(new File(archDesc));
		}
	}
	
	private void generateImplementation() {
		vmadl.checkConsistency();
		vmadl.generateImplementation(outputDir);
	}

	private void createOptions() {
		options = new Options();
		//options.addOption("test", false, "if the test option is set, output to test generator is produced");
		//options.addOption("f", "file", true, "output will be generated into files with given prefix if this option isn't used, output will be splited up into separate files for each service.");
		//options.addOption("o", "output-path", true, "output will be generated into given folder");
		options.addOption("?", "help", false, "prints this help message");
	}
	
	private void printUsage() {
		HelpFormatter formatter = new HelpFormatter();
		formatter.printHelp("vmadl <input-file1> [... <input-fileN>] <output-folder>",
				"vmadl will compile the given VM architecture description into c-header and AspectC++-aspect files.", options, "", true);
		
	}
}


