grammar Vmadl;

options {
	output = AST;
  //ASTLabelType=pANTLR3_COMMON_TREE;
  language = Java;
  backtrack = true;
}

tokens {
    SERVICE		= 'service';
    STARTUP		= 'startup';
    SHUTDOWN	= 'shutdown';
    EXPOSE		= 'expose';
    ADVICE		= 'advice';
    POINTCUT	= 'pointcut';
    REQUIRE		= 'require';
    BOOT		  = 'boot'; // would be nice if i could get rid of this
    FOR				= 'for';
}

@header {
  package de.sm.vmadl;
  
  import de.sm.vmadl.elements.*;
}

@lexer::header {
  package de.sm.vmadl;
}

@members {
	private ServiceRegistry reg;
}

architecture [ServiceRegistry reg] returns [ List<ServiceModule> modules ] :
{ 
	$modules = new ArrayList<ServiceModule>();
	this.reg = $reg;
}
    /* Include only service nodes into tree, additional declarations are handled using raw data */
    ( //additionalDeclaration! |
     NEWLINE!
     | service { $modules.add($service.s);}
     )* EOF!
;

/*additionalDeclaration 
  : (~(LBRACE | RBRACE | STARTUP | EXPOSE | SERVICE))+
;*/

service returns [ServiceModule s] :
 	SERVICE SERVICE_NAME { $s = reg.get($SERVICE_NAME.text); }
  (LBRACE 
  	(  require			{ $s.requirements.add($require.service); }
  	 | startup
  	 | shutdown
  	 | expose				{ $s.pointcuts = $expose.pointcuts; }
  	 | NEWLINE
  	 | advice				{ $s.advices.add($advice.text); }
  	 | declarations { $s.declarations.add($declarations.text); }
  	 | optionalRelations { $s.relations.add($optionalRelations.s); }
  	)* RBRACE)
  { $s.initialized = true; }
;

advice :
	ADVICE adviceHeader content
;

adviceHeader :
	 ( ~(LBRACE | RBRACE | ';') )+
;

declarations :
	preprocessor | declaration
;

preprocessor :
  '#'
  (options {greedy=false;} : (~(LBRACE | RBRACE | ';' | NEWLINE) )+ )
   NEWLINE		// may be this is not the best way, but will work for now
;

    

/*declaration :
	(options {greedy=false;} : ~('#' | NEWLINE | LBRACE | RBRACE | SEMICOLON | SERVICE | STARTUP | SHUTDOWN | EXPOSE | ADVICE | POINTCUT | REQUIRE | BOOT ) )
	(options {greedy=false;} : ( ~(LBRACE | RBRACE | SEMICOLON) )+ )
	SEMICOLON NEWLINE?
; */

declaration :
	(~('#' | NEWLINE | LBRACE | RBRACE | ';' | SERVICE | STARTUP | SHUTDOWN | EXPOSE | ADVICE | POINTCUT | REQUIRE | BOOT | FOR ))
	
	( ~(LBRACE | RBRACE | ';') )+
	content?	
	';'?
;

content
  : //LBRACE content? ( options {greedy=false;} : . )* RBRACE
    LBRACE
	(content | (~(LBRACE | RBRACE))+)*
	RBRACE
;

startup
  : STARTUP
	(LBRACE (expose | advice | boot )* RBRACE);

shutdown
  : SHUTDOWN
	(LBRACE (expose | advice)* RBRACE);	
	
boot :
	BOOT
	( ~(LBRACE | RBRACE | ';') )+
	';'
;
	
expose returns [List<String> pointcuts] :
{ $pointcuts = new ArrayList<String>(); }
  EXPOSE NEWLINE* LBRACE NEWLINE* 
	( pointcut {$pointcuts.add($pointcut.text); }
	)*
	RBRACE
;

require returns [ ServiceModule service ] : 
	REQUIRE SERVICE_NAME { $service = reg.get($SERVICE_NAME.text); } 
	';';
	
pointcut :
	POINTCUT 
	(~(';'))*
	';' NEWLINE* 
;

optionalRelations returns [OptionalRelation s] :
	FOR SERVICE_NAME { $s = new OptionalRelation(); 
										 $s.service = reg.get($SERVICE_NAME.text); }
	(LBRACE 
  	(  require			{ $s.requirements.add($require.service); }
  	 | startup
  	 | shutdown
  	 | expose				{ $s.pointcuts = $expose.pointcuts; }
  	 | NEWLINE
  	 | advice				{ $s.advices.add($advice.text); }
  	 | declarations { $s.declarations.add($declarations.text); }
  	)* RBRACE)
;

SERVICE_NAME : LETTER ( DIGIT | LETTER | '-' | '_' )*;

DIGIT
    :    '0'..'9'
    ;
fragment LETTER
    : 'a'..'z'
    | 'A'..'Z'
    ;

SPECIAL : '(' | ')' | ':' | '='| '.' | '*' | '+' | '%' | '#' | '/' | '\\'
	| '_' | ';' | ',' | '|' | '&' | '-' | '"' | '<' | '>' | '[' | ']' | '!' | '\'' ;

NEWLINE
  : '\r'? '\n'
  //{ channel=HIDDEN; }
;

WS
  : (' '|'\t')+		//|'\n'|'\r' 
  { channel=HIDDEN; }
;

COMMENT
  : '//' (~('\n'|'\r'))* NEWLINE
  { channel=HIDDEN; }
;
  
// multiple-line comments
ML_COMMENT
  : '/*' (options {greedy=false;} : .)* '*/' 
  { channel=HIDDEN; }
;

//SEMICOLON : ';';
LBRACE : '{';
RBRACE : '}';
