package de.sm.vmadl.elements;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.antlr.runtime.ParserRuleReturnScope;

/**
 * A combination is a full featured programming unit.
 * It can contain aspects, but usual constructs from the 
 * implementation language as well.
 * 
 * @author Stefan Marr
 */
public class Combination extends ArchitectureElement {
	
	public Combination(CombinationId id) {
		this(id, null, null);
	}
	
	public Combination(CombinationId id, ParserRuleReturnScope prrs, File file) {
		super(prrs, file);
		this.id = id;
		advices = new ArrayList<Advice>();
		startupAdvices = new ArrayList<Advice>();
		shutdownAdvices = new ArrayList<Advice>();
		functions = new ArrayList<Function>();
		variables = new ArrayList<Variable>();
		preprocessor = new ArrayList<String>();
	}

	public CombinationId id;

	/**
	 * Raw data string with advices from a service.
	 */
	public List<Advice> advices;
	
	/**
	 * Raw data string with service advices, only active in startup phase. 
	 */
	public List<Advice> startupAdvices;
	
	/**
	 * Raw data string with service advices, only active in shutdown phase. 
	 */
	public List<Advice> shutdownAdvices;
	
	public List<Function> functions;
	
	public List<Variable> variables;
	
	public List<String> preprocessor;

}
