package de.sm.vmadl.elements;

import java.io.File;

import org.antlr.runtime.ParserRuleReturnScope;
import org.antlr.runtime.Token;

/**
 * Is the base class for all language elements part of the semi-AST.
 * 
 * @author Stefan Marr
 */
public class LanguageElement {

	private ParserRuleReturnScope returnValue;
	private File file;
	
	/**
	 * Initialize element with basic file information.
	 * @param prrs ANTLR information about associated tokens
	 * @param file in which this definition was parsed
	 */
	public LanguageElement(final ParserRuleReturnScope prrs, final File file) {
		returnValue = prrs;
		this.file = file;
	}
	
	public final Token getStart() {
		return returnValue.start;
	}
	
	public final Token getStop() {
		return returnValue.stop;
	}
	
	public final File getFile() {
		return file;
	}
	
	public final void lazyInitWith(final ParserRuleReturnScope prrs,
			final File file) {
		if (returnValue != null || this.file != null) {
			throw new IllegalStateException(
					"Instance has been inited already." + this);
		}
		returnValue = prrs;
		this.file = file;
	}
}
