package de.sm.vmadl.elements;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.sm.odl.elements.Type;

/**
 * Basic service type, includes all definitions necessary to describe
 * parts of a service. For a full service module {@see ServiceModule}
 * 
 * @author Stefan Marr
 */
public class ServiceInterface extends ArchitectureElement {
	
	/**
	 * Initializes public list fields.
	 */
	public ServiceInterface() {
		super(null, null); // TODO: add Support for language element
		requirements = new ArrayList<ServiceModule>();
		declarations = new ArrayList<String>();
		pointcuts = new ArrayList<String>();
		startupPointcuts = new ArrayList<String>();		
		shutdownPointcuts = new ArrayList<String>();
		types = new ArrayList<Type>();
		parts = new HashMap<QualifiedIdentifier, ServiceInterface>();
		replacements = new HashMap<QualifiedIdentifier, ServiceInterface>();
	}
	
	/**
	 * Raw data strings with includes, global variables and function
	 * definitions. Advices are stored in their own field.
	 */
	public List<String> declarations;
		
	/**
	 * Raw data string with pointcuts exposed by a service.
	 */
	public List<String> pointcuts;
	
	/**
	 * Raw data string with service pointcuts, only active in startup phase.
	 */
	public List<String> startupPointcuts;
	
	/**
	 * Raw data string with service pointcuts, only active in shutdown phase.
	 */
	public List<String> shutdownPointcuts;
	
	/**
	 * Text of the types section, has to be parsed separately.
	 */
	public String typesSection;
	
	/**
	 * Holds the types information gathered from the types section.
	 */
	public List<Type> types;
	
	/**
	 * If this is only a part of a parent definition this partName is set.
	 */
	public QualifiedIdentifier partName;
	
	/**
	 * List of named subparts of this module
	 * identified with a qualified id.
	 */
	public Map<QualifiedIdentifier, ServiceInterface> parts;
	
	/**
	 * List of parts which define replacements for other modules.
	 * They are accessible by a qualified id consisting of a module name
	 * and the name of the named part to be replaced 
	 */
	public Map<QualifiedIdentifier, ServiceInterface> replacements;
}
