#ifndef VMIMAGE_H_
#define VMIMAGE_H_

/*
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
  */

#include <vm/Image.h>

#include <vmobjects/VMObject.h>
#include <vmobjects/VMString.h>

#pragma mark VTable definition

VTABLE(VMImage) {
#define VMIMAGE_VTABLE_FORMAT \
    VMOBJECT_VTABLE_FORMAT; \
    ImageHeader       (*get_image_header)(void*); \
    ImageInfoHeader   (*get_info_header)(void*); \
    pVMString         (*get_file_name)(void*); \
    pVMInteger        (*get_size)(void*); \
    time_t            (*get_timestamp)(void*); \
    int32_t           (*get_version)(void*); \
    int32_t           (*get_word_length)(void*); \
    bool              (*is_compressed)(void*); \
    bool              (*is_encrypted)(void*); \
    bool              (*is_loadable)(void*)
        
    VMIMAGE_VTABLE_FORMAT;
};

#pragma mark class definition

struct _VMImage {
    VTABLE(VMImage)* _vtable;

#define IMAGE_FORMAT \
    VMOBJECT_FORMAT; \
    pVMString file_name; \
    pVMInteger image_size; \
    ImageHeader image_header; \
    ImageInfoHeader info_header
    
    IMAGE_FORMAT;
};

#pragma mark class methods

pVMImage VMImage_new(void);
pVMImage VMImage_new_with(const pVMInteger file_size, const pVMString file_name, const ImageHeader image_header,
		const ImageInfoHeader info_header);

#pragma mark vtable initialization

VTABLE(VMImage)* VMImage_vtable(void);

#endif VMIMAGE_H_


