/* Native multi-threading for CSOM - Thread primitive class
 * 
 * Copyright (C) 2007 by Martin Beck and Jan Klimke
 * 
 * created 2007-05-09
 */
 
#include "VMThread.h"

#include <vm/Universe.h>

#include <memory/memory.h>

#include <sched.h>

ThreadId (*get_embedded_thread_id)(pVMThread);
void     (*set_embedded_thread_id)(pVMThread, ThreadId);
void     (*join)(pVMThread, int*);

pVMThread VMThread_new() {
	pVMThread result = (pVMThread) memory_allocate(sizeof(VMThread));
    if(result) {
        result->_vtable = VMThread_vtable();
        INIT(result, NUMBER_OF_OBJECT_FIELDS(VMThread));
    }
    SEND(result, set_class, thread_class);
    return result;
}


pVMSignal _VMThread_get_resume_signal(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->resume_signal;
}


void _VMThread_set_resume_signal(void* _self, pVMSignal value) {
    pVMThread self = (pVMThread)_self;
    
    self->resume_signal = value;
}


bool _VMThread_should_stop(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->should_stop == true_object;
}


void _VMThread_set_should_stop(void* _self, bool value) {
    pVMThread self = (pVMThread)_self;
    
    if (value) {
    	self->should_stop = true_object;
    } else {
    	self->should_stop = false_object;
    }
}

pVMBlock _VMThread_get_block_to_run(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->block_to_run;
}


void _VMThread_set_block_to_run(void* _self, pVMBlock value) {
    pVMThread self = (pVMThread)_self;
    
    self->block_to_run = value;
}


pVMObject _VMThread_get_argument(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->argument;
}


void _VMThread_set_argument(void* _self, pVMObject value) {
    pVMThread self = (pVMThread)_self;
    
    self->argument = value;
}


pVMString _VMThread_get_name(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->name;
}


void _VMThread_set_name(void* _self, pVMString value) {
    pVMThread self = (pVMThread)_self;
    
    self->name = value;
}


ThreadId _VMThread_get_embedded_thread_id(pVMThread i) {
	return i->embedded_thread_id;
}

void _VMThread_set_embedded_thread_id(pVMThread i, ThreadId value) {
	i->embedded_thread_id = value;
}

void VMThread_create_safe_global(ThreadSafeGlobal *global) {
	pthread_key_create(global, NULL);
}

void VMThread_delete_safe_global(ThreadSafeGlobal global) {
	pthread_key_delete(global);
}

void* VMThread_get_safe_global(ThreadSafeGlobal global) {
	return pthread_getspecific(global);
}

void VMThread_set_safe_global(ThreadSafeGlobal global, void* value) {
	pthread_setspecific(global, value);
}

void _VMThread_join(pVMThread self, int *exit_status) {
	pthread_join(self->embedded_thread_id, (void **)exit_status);
}

void VMThread_yield() {
	sched_yield();
}

static VTABLE(VMThread) _VMThread_vtable;
bool   VMThread_vtable_inited = false;


VTABLE(VMThread)* VMThread_vtable(void) {
    if(VMThread_vtable_inited) {
    	return &_VMThread_vtable;
    }
    
    *((VTABLE(VMObject)*)&_VMThread_vtable) = *VMObject_vtable();
    _VMThread_vtable.get_resume_signal = METHOD(VMThread, get_resume_signal);
    _VMThread_vtable.set_resume_signal = METHOD(VMThread, set_resume_signal);
    _VMThread_vtable.should_stop = METHOD(VMThread, should_stop);
    _VMThread_vtable.set_should_stop = METHOD(VMThread, set_should_stop);
    _VMThread_vtable.get_block_to_run = METHOD(VMThread, get_block_to_run);
    _VMThread_vtable.set_block_to_run = METHOD(VMThread, set_block_to_run);
    _VMThread_vtable.get_argument = METHOD(VMThread, get_argument);
    _VMThread_vtable.set_argument = METHOD(VMThread, set_argument);
    _VMThread_vtable.get_name = METHOD(VMThread, get_name);
    _VMThread_vtable.set_name = METHOD(VMThread, set_name);
    _VMThread_vtable.get_embedded_thread_id = METHOD(VMThread, get_embedded_thread_id);
    _VMThread_vtable.set_embedded_thread_id = METHOD(VMThread, set_embedded_thread_id);
    _VMThread_vtable.join = METHOD(VMThread, join);
    
    VMThread_vtable_inited = true;

    return &_VMThread_vtable;
}
