/*
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
  */

#include "Block.h"

#include <interpreter/bytecodes.h>
#include <interpreter/Interpreter.h>

#include <native/VMThread.h>

#include <vmobjects/VMObject.h>
#include <vmobjects/VMClass.h>
#include <vmobjects/VMFrame.h>
#include <vmobjects/VMInvokable.h>

#include <vm/Universe.h>

extern "C" {

void  _Block_value(pVMObject object, pVMFrame frame) {
    // intentionally left blank
}


void  _Block_value_(pVMObject object, pVMFrame frame) {
    // intentionally left blank
}


void  _Block_value_with_(pVMObject object, pVMFrame frame) {
    // intentionally left blank
}


void  _Block_restart(pVMObject object, pVMFrame frame) {
    SEND(frame, set_bytecode_index, 0);
    SEND(frame, reset_stack_pointer);    
}

pVMMethod create_fake_bootstrap_method() {
	return Interpreter_create_bootstrap_method(Universe_get_block_class());
	/*pVMMethod bootstrapVMMethod = Universe_new_method(Universe_symbol_for_chars("bootstrap"), 1, 0);
	SEND(bootstrapVMMethod, set_bytecode, 0, BC_HALT);
	SEND(bootstrapVMMethod, set_number_of_locals, 0);
	SEND(bootstrapVMMethod, set_maximum_number_of_stack_elements, 1); //TODO: if anything goes wrong, universe uses 2 instead of 1 
	TSEND(VMInvokable, bootstrapVMMethod, set_holder, (pVMClass)Universe_get_block_class());
	return bootstrapVMMethod;*/
}

// Native Multithreading by Martin Beck and Jan Klimke                                                                      
void *threadForBlock(void* thread_pointer) {                                                                                                                           
	pVMThread thread = (pVMThread)thread_pointer;                                                                          
	pVMBlock block = (pVMBlock)SEND(thread, get_block_to_run);
                                                                                                                        
    //set the thread object for the interpreter                                                                          
    Interpreter_set_thread(thread);
                                                                                                                    
    // create a fake bootstrap method to simplify later frame traversal                                                  
    pVMMethod bootstrapVMMethod = create_fake_bootstrap_method();
                                                                                                                        
    // create a fake bootstrap frame with the block object on the stack                                                     
    pVMFrame bootstrapVMFrame;
    Interpreter_push_new_frame(bootstrapVMMethod, &bootstrapVMFrame);
    SEND(bootstrapVMFrame, push, (pVMObject)block);                                                                     
                                                                                                                        
    // lookup the initialize invokable on the system class                                                                  
    pVMInvokable initialize = (pVMInvokable)SEND((pVMClass)Universe_get_block_class(), lookup_invokable, Universe_symbol_for_chars("evaluate"));                                                                                               

    // invoke the initialize invokable
    TSEND(VMInvokable, initialize, invoke, bootstrapVMFrame);
                                                                                                                    
    // start the interpreter                                                                                             
    Interpreter_start();
                                                                                                                    
    // exit this thread                                                                                                  
	pthread_exit(NULL);                                                                                                  
}

void *threadForBlockWithArgument(void* thread_pointer) {                                                                                                                           
	pVMThread thread = (pVMThread)thread_pointer;                                                                          
    pVMBlock block = (pVMBlock)SEND(thread, get_block_to_run);
                                                                                                                        
    // set the thread object for the interpreter                                                                          
    Interpreter_set_thread(thread);                                                                                
                                                                                                                    
    // create a fake bootstrap method to simplify later frame traversal                                                  
    pVMMethod bootstrapVMMethod = create_fake_bootstrap_method();
                                                                                                                        
    // create a fake bootstrap frame with the block object on the stack                                                     
    pVMFrame bootstrapVMFrame;
    Interpreter_push_new_frame(bootstrapVMMethod, &bootstrapVMFrame);
    SEND(bootstrapVMFrame, push, (pVMObject)block);
    pVMObject arg = (pVMObject)SEND(thread, get_argument);
    SEND(bootstrapVMFrame, push, arg);           
                                                                                                                        
    // lookup the initialize invokable on the system class                                                                  
    pVMInvokable initialize = (pVMInvokable)SEND((pVMClass)Universe_get_block_class(), lookup_invokable, Universe_symbol_for_chars("evaluate:"));                                                                                              
                                                                                                    
    // invoke the initialize invokable                                                                                      
    TSEND(VMInvokable, initialize, invoke, bootstrapVMFrame);                                                                    
                                                                                                                    
    // start the interpreter                                                                                             
    Interpreter_start();                                                                                              
                                                                                                                    
    // exit this thread                                                                                                  
	pthread_exit(NULL);                                                                                                  
}

pVMThread create_new_thread(pVMBlock block) {
    pVMThread thread = VMThread_new();
    // set the Object fields of the thread object
    SEND(thread, set_resume_signal, VMSignal_new());
    SEND(thread, set_should_stop, false);
    SEND(thread, set_block_to_run, block);
    return thread;
}

void _Block_spawn(pVMObject object, pVMFrame frame) {                                                                         
	ThreadId tid = 0;
    pVMBlock block = (pVMBlock)SEND(frame, pop);
    
    // create the thread object                                                                                           
    pVMThread thread = create_new_thread(block);
    
    //Create the pthread
	pthread_create(&tid, NULL, threadForBlock, (void*)thread);
	SEND(thread, set_embedded_thread_id, tid);
	SEND(frame, push, (pVMObject)thread);
}                                                                                                                           
                                                                                                                            
void _Block_spawnWithArgument_(pVMObject object, pVMFrame frame) {                                                             
	ThreadId tid = 0;                                                                                                   
	
	// Get the argument                                                                                                   
    pVMObject argument = SEND(frame, pop);                                                                          
    pVMBlock block = (pVMBlock)SEND(frame, pop);                                                                     
    
    // create the thread object                                                                                           
    pVMThread thread = create_new_thread(block);
    SEND(thread, set_argument, argument);
    
    //Create the pthread                                                                                                 
	pthread_create(&tid, NULL, threadForBlockWithArgument, (void *)thread);                                              
    SEND(thread, set_embedded_thread_id, tid);                                                                    
    SEND(frame, push, (pVMObject)thread);                                                                            
}

}

