/* 
 * Thread.c
 *  
 * Native multi-threading for CSOM
 * 
 * by Martin Beck and Jan Klimke
 * 
 * created 2007-05-10
 */

#include "Thread.h"

#include <vmobjects/VMObject.h>
#include <vmobjects/VMFrame.h>
#include <vmobjects/VMInteger.h>

#include <native-threads/native.h>
#include <native-threads/VMThread.h>

#include <module.Interpreter>

#include <pthread.h>
#include <sched.h>

extern "C" {

void _Thread_join(pVMObject object, pVMFrame frame){
	pVMThread thread = (pVMThread)SEND(frame, pop);
	int return_value;	
	pthread_t threadid = SEND(thread, get_embedded_thread_id);
    SEND(thread, join, &return_value);
    SEND(frame, push, (pVMObject)thread);
}

void _Thread_priority_(pVMObject object, pVMFrame frame){
	// get the new priority integer object
    pVMInteger arg = (pVMInteger) SEND(frame, pop);
    int prio = SEND(arg, get_embedded_integer);
    pVMThread thread = (pVMThread)SEND(frame, pop);
    ThreadId threadid = SEND(thread, get_embedded_thread_id);
    //nobody knows what values to insert for Priority
    // it depends on the scheduler implementation
    //int prioret = pthread_setschedprio(threadid,prio);
    
    int policy;
    struct sched_param param;
    pthread_getschedparam(threadid, &policy, &param);
    param.sched_priority = prio;
    pthread_setschedparam(threadid, policy, &param);
    
    SEND(frame, push, (pVMObject)thread);
}

void Thread_yield(pVMObject object, pVMFrame frame) {
	pVMThread thread = (pVMThread)SEND(frame, pop);
    VMThread_yield();
    SEND(frame, push, (pVMObject)thread);
}

void Thread_current(pVMObject object, pVMFrame frame) {
	pVMThread thread = (pVMThread)SEND(frame, pop);
    SEND(frame, push, (pVMObject)VMThread_get_safe_global(tsg_frame));
}

}
