#!/usr/bin/env make -f

#
#  Makefile by Tobias Pape
#
# Copyright (c) 2007 Michael Haupt, Tobias Pape
# Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
# http://www.hpi.uni-potsdam.de/swa/
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

##############
############## Collections.

OBJECTS			= $(CSOM_OBJ) $(PRIMITIVES_OBJ) $(NATIVE_PRIM_OBJ) $(GREEN_PRIM_OBJ)

SOURCES			= $(COMPILER_SRC) $(INTERPRETER_SRC) $(MEMORY_SRC) \
					$(MISC_SRC) $(VM_SRC) $(VMOBJECTS_SRC)  \
					$(PRIMITIVES_SRC) $(GREEN_SRC) $(THREADED_SRC)

SOURCES_ACC		= $(COMPILER_ACC) $(INTERPRETER_ACC) $(MEMORY_ACC) \
					$(MISC_ACC) $(VM_ACC) $(VMOBJECTS_ACC)  \
					$(PRIMITIVES_ACC) $(GC_MS_ACC) $(GC_RC_ACC) $(NATIVE_ACC) $(NATIVE_PRIM_ACC) \
					$(GREEN_ACC) $(GREEN_PRIM_ACC) $(THREADED_ACC)

############# Things to clean

CLEAN			= $(OBJECTS) CORE $(SOURCES_ACC) $(VMADL_GEN_DIR)/module.* vmadl

CLOBBER_CFG		= modules.obj modules.prim modules.som modules.libs modules.vmadl modules.src modules.cflags

############# Tools

OSTOOL			= $(BUILD_DIR)/ostool.exe

CORE			= $(BIN_ST_DIR)/$(CORE_NAME).csp

#
#
#
#  metarules
#

.SUFFIXES: .pic.o

all: prepare $(BIN_DIR)/CSOM $(CORE)


debug : DBG_FLAGS=-DDEBUG -g3
debug: all

profiling : DBG_FLAGS=-g -pg
profiling : LDFLAGS+=-pg
profiling: all

%.o: %.c
	$(COMPILE.c) $(OUTPUT_OPTION) $<

.c.pic.o:
	$(CC) $(CFLAGS) -fPIC -g3 -c $< -o $*.pic.o


clean:
	@echo clean: Delete object files, core, and woven acc-files
	@rm -Rf $(CLEAN)


clobber: $(OSTOOL) clean
	@echo clobber: Delete CSOM, OSTOOL, and clean
	@rm -f `$(OSTOOL) x "$(CSOM_NAME)"` $(ST_DIR)/$(CORE_NAME).csp
	@rm -f $(OSTOOL)
	@rm -f $(CLOBBER_CFG)
	@rm -f $(SRC_DIR)/platform.h
	@rm -Rf $(VMADL_GEN_DIR)
	@rm -Rf $(BIN_DIR)

$(OSTOOL): $(BUILD_DIR)/ostool.c
	cc -g -o $(OSTOOL) $(BUILD_DIR)/ostool.c

$(SRC_DIR)/platform.h: $(OSTOOL)
	@($(OSTOOL) i >$(SRC_DIR)/platform.h)

#
#
#
# product rules
#

vmadl:
	@echo Compile VMADL files
	@java -jar $(BUILD_DIR)/vmadl/vmadl.jar $(VMADL) `cat modules.vmadl` $(VMADL_GEN_DIR)
	@touch vmadl

#vmadl

$(BIN_DIR)/CSOM: prepare $(shell cat modules.obj) $(CSOM_OBJ) $(BIN_DIR)
	@echo Linking CSOM
	$(CC) $(LDFLAGS) `$(OSTOOL) l`\
		-o `$(OSTOOL) x "$(BIN_DIR)/$(CSOM_NAME)"` \
		$(CSOM_OBJ) `cat modules.obj` $(CSOM_LIBS) 
	@echo CSOM done.

$(CORE): prepare $(shell cat modules.prim) $(BIN_DIR)/CSOM $(PRIMITIVES_OBJ) $(BIN_ST_DIR) Module.som $(MODULES_PRIM_OBJ)
	@echo Linking SOMCore lib
	$(CC) $(LDFLAGS) `$(OSTOOL) l "$(CORE_NAME)"` \
		-o $(CORE) \
		$(PRIMITIVES_OBJ) `cat modules.prim` $(CORE_LIBS)
	@echo SOMCore done.
	
$(BIN_ST_DIR): $(BIN_EX_DIR) $(BIN_TEST_DIR)
	@echo Assemble Smalltalk Class Library
	@mkdir $(BIN_ST_DIR)
	@cp $(ST_DIR)/*.som $(BIN_ST_DIR)
	
$(BIN_DIR):
	@mkdir $(BIN_DIR)  

$(BIN_EX_DIR): $(BIN_DIR)
	@echo Assemble Examples
	@cp -R $(EX_DIR) $(BIN_EX_DIR)
	@find $(BIN_EX_DIR) -name '.svn' -exec rm -Rf {} +
	
$(BIN_TEST_DIR): $(BIN_EX_DIR)
	@echo Assemble Tests
	@cp -R $(TEST_DIR) $(BIN_TEST_DIR)
	@find $(BIN_TEST_DIR) -name '.svn' -exec rm -Rf {} +
	
Module.som: $(BIN_DIR) $(BIN_ST_DIR)
ifneq ($(shell cat modules.som),)
	@echo Copy Module specific runtime libraries
	cp -R `cat modules.som` $(BIN_DIR)/.
	@find $(BIN_DIR) -name '.svn' -exec rm -Rf {} +
endif

install: all
	@echo installing CSOM into build
	$(INSTALL) -d $(DEST_SHARE) $(DEST_BIN) $(DEST_LIB)
	$(INSTALL) $(BIN_DIR)/`$(OSTOOL) x "$(CSOM_NAME)"` $(DEST_BIN)
	@echo CSOM.
	cp -R $(ST_DIR) $(DEST_LIB)
	@echo Library.
	cp -R $(EX_DIR) $(TEST_DIR) $(DEST_SHARE)
	@echo shared components.
	@echo done.
	
uninstall:
	@echo removing Library and shared Components
	rm -Rf $(DEST_SHARE) $(DEST_LIB)
	@echo removing CSOM
	rm -Rf $(DEST_BIN)/`$(OSTOOL) x "$(CSOM_NAME)"`
	@echo done.

#
# test: run the standard test suite
#
test: all
	@($(BIN_DIR)/CSOM -cp $(BIN_ST_DIR) $(BIN_TEST_DIR)/TestHarness.som;)

#
# bench: run the benchmarks
#
bench: all
	@($(BIN_DIR)/CSOM -cp $(BIN_ST_DIR) $(BIN_EX_DIR)/Benchmarks/All.som;)
