#!/usr/bin/env make -f

#
#  Makefile by Tobias Pape
#
# $Id: unix.make 229 2008-04-22 14:10:50Z tobias.pape $
#
# Copyright (c) 2007 Michael Haupt, Tobias Pape
# Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
# http://www.hpi.uni-potsdam.de/swa/
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

## we need c99 not any longer ;) but Ag++!

CC		=ag++
CFLAGS		=-Wno-endif-labels $(DBG_FLAGS) $(INCLUDES) -O3 -D__STDC_LIMIT_MACROS --keep_woven `cat modules.cflags`
LDFLAGS		=$(LIBRARIES)

INSTALL		=install

CSOM_LIBS	=-ldl `cat modules.libs`
CORE_LIBS	=-lm

CSOM_NAME	=CSOM
SOM_NAME	=SOM
CORE_NAME	=$(SOM_NAME)Core

############ global stuff -- overridden by ../Makefile

PREFIX		?= /usr/local

ROOT_DIR	?= $(PWD)/..
SRC_DIR		?= $(ROOT_DIR)/src
BUILD_DIR   ?= $(ROOT_DIR)/build
BIN_DIR	    ?= $(ROOT_DIR)/bin

DEST_SHARE	?= $(PREFIX)/share/$(SOM_NAME)
DEST_BIN	?= $(PREFIX)/bin
DEST_LIB	?= $(PREFIX)/lib/$(SOM_NAME)


ST_DIR		?= $(ROOT_DIR)/Smalltalk
EX_DIR		?= $(ROOT_DIR)/Examples
TEST_DIR	?= $(ROOT_DIR)/TestSuite

MOD_SOM_DIR = $(ROOT_DIR)/Modules.som
MOD_PRIM_DIR = $(ROOT_DIR)/Modules.prim

VMADL_GEN_DIR	= $(ROOT_DIR)/generated

############# Product pathes

BIN_ST_DIR	?= $(BIN_DIR)/Smalltalk
BIN_EX_DIR	?= $(BIN_DIR)/Examples
BIN_TEST_DIR?= $(BIN_DIR)/TestSuite

############# "component" directories


COMPILER_DIR 	= $(SRC_DIR)/compiler
INTERPRETER_DIR = $(SRC_DIR)/interpreter
MEMORY_DIR 		= $(SRC_DIR)/memory
GC_RC_DIR		= $(SRC_DIR)/gc-refcount
GC_MS_DIR 		= $(SRC_DIR)/gc-marksweep
MISC_DIR 		= $(SRC_DIR)/misc
VM_DIR 			= $(SRC_DIR)/vm
VMOBJECTS_DIR 	= $(SRC_DIR)/vmobjects
NATIVE_DIR		= $(SRC_DIR)/native-threads
GREEN_DIR		= $(SRC_DIR)/green-threads
TAGINT_ONE_DIR 	= $(SRC_DIR)/tagged-int-one
THREADED_DIR 	= $(SRC_DIR)/interpreter-threaded

COMPILER_SRC	= $(wildcard $(COMPILER_DIR)/*.c)
COMPILER_OBJ	= $(COMPILER_SRC:.c=.o)
COMPILER_ACC	= $(COMPILER_SRC:.c=.acc)
INTERPRETER_SRC	= $(wildcard $(INTERPRETER_DIR)/*.c)
INTERPRETER_OBJ	= $(INTERPRETER_SRC:.c=.o)
INTERPRETER_ACC	= $(INTERPRETER_SRC:.c=.acc)
THREADED_SRC	= $(wildcard $(THREADED_DIR)/*.c)
THREADED_OBJ	= $(THREADED_SRC:.c=.o)
THREADED_ACC	= $(THREADED_SRC:.c=.acc)
MEMORY_SRC		= $(wildcard $(MEMORY_DIR)/*.c)
MEMORY_OBJ		= $(MEMORY_SRC:.c=.o)
MEMORY_ACC		= $(MEMORY_SRC:.c=.acc)
GC_RC_SRC		= $(wildcard $(GC_RC_DIR)/*.c)
GC_RC_OBJ		= $(GC_RC_SRC:.c=.o)
GC_RC_ACC		= $(GC_RC_SRC:.c=.acc)
GC_MS_SRC		= $(wildcard $(GC_MS_DIR)/*.c)
GC_MS_OBJ		= $(GC_MS_SRC:.c=.o)
GC_MS_ACC		= $(GC_MS_SRC:.c=.acc)
MISC_SRC		= $(wildcard $(MISC_DIR)/*.c)
MISC_OBJ		= $(MISC_SRC:.c=.o)
MISC_ACC		= $(MISC_SRC:.c=.acc)
VM_SRC			= $(wildcard $(VM_DIR)/*.c) $(SRC_DIR)/main.c
VM_OBJ			= $(VM_SRC:.c=.o)
VM_ACC			= $(VM_SRC:.c=.acc)
VMOBJECTS_SRC	= $(wildcard $(VMOBJECTS_DIR)/*.c)
VMOBJECTS_OBJ	= $(VMOBJECTS_SRC:.c=.o)
VMOBJECTS_ACC	= $(VMOBJECTS_SRC:.c=.acc)
NATIVE_SRC		= $(wildcard $(NATIVE_DIR)/*.c)
NATIVE_OBJ		= $(NATIVE_SRC:.c=.o)
NATIVE_ACC		= $(NATIVE_SRC:.c=.acc)
GREEN_SRC		= $(wildcard $(GREEN_DIR)/*.c)
GREEN_OBJ		= $(GREEN_SRC:.c=.o)
GREEN_ACC		= $(GREEN_SRC:.c=.acc)
TAGINT_ONE_SRC	= $(wildcard $(TAGINT_ONE_DIR)/*.c)
TAGINT_ONE_OBJ	= $(TAGINT_ONE_SRC:.c=.o)
TAGINT_ONE_ACC	= $(TAGINT_ONE_SRC:.c=.acc)

NATIVE_SOM_DIR	= $(MOD_SOM_DIR)/native-threads
NATIVE_PRIM_DIR	= $(MOD_PRIM_DIR)/native-threads
GREEN_SOM_DIR	= $(MOD_SOM_DIR)/green-threads
GREEN_PRIM_DIR	= $(MOD_PRIM_DIR)/green-threads

############# primitives location etc.

PRIMITIVES_DIR	= $(SRC_DIR)/primitives
PRIMITIVES_SRC	= $(wildcard $(PRIMITIVES_DIR)/*.c)
PRIMITIVES_OBJ	= $(PRIMITIVES_SRC:.c=.pic.o)
PRIMITIVES_ACC	= $(PRIMITIVES_SRC:.c=.acc)

NATIVE_PRIM_SRC	= $(wildcard $(NATIVE_PRIM_DIR)/*.c)
NATIVE_PRIM_OBJ	= $(NATIVE_PRIM_SRC:.c=.pic.o)
NATIVE_PRIM_ACC	= $(NATIVE_PRIM_SRC:.c=.acc)
GREEN_PRIM_SRC	= $(wildcard $(GREEN_PRIM_DIR)/*.c)
GREEN_PRIM_OBJ	= $(GREEN_PRIM_SRC:.c=.pic.o)
GREEN_PRIM_ACC	= $(GREEN_PRIM_SRC:.c=.acc)

############# VMADL architecture description for the basic system

VMADL := $(SRC_DIR)/Bytecode.vmadl $(SRC_DIR)/interpreter.vmadl $(SRC_DIR)/memory.vmadl $(SRC_DIR)/vmobjects.vmadl $(SRC_DIR)/vm.vmadl 

############# Collections

CSOM_OBJ 		= $(MEMORY_OBJ) $(MISC_OBJ) $(VMOBJECTS_OBJ) \
					$(COMPILER_OBJ) $(INTERPRETER_OBJ) $(VM_OBJ)
					
MOD_SOM_CP		=

############# include path

INCLUDES		=-I$(VMADL_GEN_DIR) -I$(SRC_DIR)
LIBRARIES       =
