#ifndef GC_H_
#define GC_H_

/*
 * $Id: gc.h 250 2008-04-28 10:44:12Z michael.haupt $
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */
 
#include <stdlib.h>
#include <misc/defs.h>

#include <vmobjects/OOObject.h>

void*  gc_allocate(size_t size);
void*  gc_allocate_object(size_t size);
char*  gc_allocate_string(const char* restrict str);
void   gc_free(void* ptr);

// increase reference counter of an object 
void gc_dec_reference_counter(pVMObject object);

// decrease reference counter of an object
void gc_inc_reference_counter(pVMObject object);

// set an object as global object, so it is not considered by reference counting 
void gc_set_global_object(pVMObject object);

// returns the number of allocated objects
int gc_get_number_of_allocated_objects(void);

// return the number of allocated objects minus deleted objects
int gc_get_number_of_live_objects(void);

// return the number of immortal objects
int gc_get_number_of_immortal_objects(void);

void gc_prepare_statistics(void);
void gc_statistics(void);

#endif GC_H_
