
#include <module.Bytecode>
#include <module.Interpreter>


#include <interpreter/Profiler.h>


#include "Interpreter_threaded.h"

#include <module.Memory>

#include <module.VM>

#include <module.VMObjects>

#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>

static pVMMethod method;

int interpreter_super_inst_config_mask;

void Interpreter_threaded_set_frame(pVMFrame _frame);

//#define USE_PROFILING
// convenience macros for frequently used function invocations
#define _FRAME Interpreter_get_frame()
#define _SETFRAME(f) Interpreter_threaded_set_frame(f)
#define _METHOD Interpreter_get_method()
#define _SELF Interpreter_get_self()

void** pc = NULL;

void** pc_from_index(pVMMethod _method, int index) {
	uint8_t* bytecodes = ((uint8_t*)&_method->fields[SEND(_method, _get_offset) + 
                                                     SEND(_method, get_number_of_indexable_fields)
                                                    ]);
    
    void** _pc = (void**)&(bytecodes[SEND(_method, get_number_of_bytecodes)]) + index;
    return _pc;
}

int index_from_pc(pVMMethod _method, void** _pc) {
    uint8_t* bytecodes = ((uint8_t*)&_method->fields[SEND(_method, _get_offset) + 
                                                     SEND(_method, get_number_of_indexable_fields)
                                                     ]);
    
    void** base = (void**)&(bytecodes[SEND(_method, get_number_of_bytecodes)]);
     // TODO: consider 64bit
    int index = (int)(_pc - base);
    return index;
}

#pragma mark private bytecode handlers.

pVMFrame pop_frame_threaded(void) {
    // Save a reference to the top frame
    pVMFrame result = _FRAME;
    
    // Pop the top frame from the frame stack
    _SETFRAME(SEND(_FRAME, get_previous_frame));

    // Destroy the previous pointer on the old top frame
    SEND(result, clear_previous_frame);

    // Return the popped frame
    return result;
}

void pop_frame_and_push_result_threaded(pVMObject result) {
    // Pop the top frame from the interpreter frame stack and compute the number
    // of arguments
    pVMFrame prev_frame = pop_frame_threaded();
    pVMMethod method = SEND(prev_frame, get_method);
    int number_of_arguments = SEND(method, get_number_of_arguments);
        
    // Pop the arguments
    for(int i = 0; i < number_of_arguments; i++)
        SEND(_FRAME, pop);
    
    // Push the result
    SEND(_FRAME, push, result);
}

void send_threaded(pVMSymbol signature, pVMClass receiver_class) {
    static pString dnu = NULL;
    if (!dnu) dnu = String_new("doesNotUnderstand:arguments:");
    
    // Lookup the invokable with the given signature
    pVMObject invokable = (pVMObject)SEND(receiver_class,
                                          lookup_invokable, signature);

    if(invokable != NULL)
        // Invoke the invokable in the current frame
        TSEND(VMInvokable, invokable, invoke, _FRAME);           
    else { 
        // doesNotUnderstand
        // Compute the number of arguments
        int number_of_arguments = Signature_get_number_of_arguments(signature);
        
        // Compute the receiver
        pVMObject receiver = 
            SEND(_FRAME, get_stack_element, number_of_arguments - 1);
        
        // Allocate an array with enough room to hold all arguments
        pVMArray arguments_array = VMArray_new(number_of_arguments);
      
        // Remove all arguments and put them in the freshly allocated array
        for(int i = number_of_arguments - 1; i >= 0; i--) {
            pVMObject o = SEND(_FRAME, pop);
            SEND(arguments_array, set_indexable_field, i, o);
        }

        // Send 'doesNotUnderstand:arguments:' to the receiver object
        pVMObject arguments[] =
            { (pVMObject)signature, (pVMObject)arguments_array };
        SEND(receiver, send, dnu, arguments, 2);
    }
} 

#pragma mark extern callable Interpreter funtions

/*******************
* Profiling macros *
*******************/

#ifdef USE_PROFILING

#define PROFILE(BYTECODE) \
	Profiler_add(BYTECODE)

#define PRINT_PROFILE \
	Profiler_print_html("profile.html");\
	Profiler_print_top(20);\
	Profiler_close()

#else

#define PROFILE(BYTECODE)
#define PRINT_PROFILE 

#endif // USE_PROFILING

/*****************
* Utility macros *
*****************/

#define UNPACK1(PC, DST) \
    DST = (uint8_t)(int)*PC++

#define UNPACK2(PC, DST1, DST2) \
    DST1 = (uint8_t)(int)*PC++; \
    DST2 = (uint8_t)(int)*PC++

#define DISPATCH \
if (dump_bytecodes > 1) \
    Disassembler_dump_bytecode(_FRAME, method, index_from_pc(method, pc)); \
goto **pc++

/*********************
* Instruction macros *
*********************/

#define OP_HALT { \
	PRINT_PROFILE;\
	return; \
}

#define OP_DUP { \
    pVMObject elem = SEND(_FRAME, get_stack_element, 0); \
    SEND(_FRAME, push, elem); \
}

#define OP_PUSH_LOCAL { \
    uint8_t bc1; \
    uint8_t bc2; \
    UNPACK2(pc, bc1, bc2); \
    pVMObject local = SEND(_FRAME, get_local, bc1, bc2); \
    SEND(_FRAME, push, local); \
}

#define OP_PUSH_ARGUMENT { \
    uint8_t bc1; \
    uint8_t bc2; \
    UNPACK2(pc, bc1, bc2); \
    pVMObject argument = SEND(_FRAME, get_argument, bc1, bc2); \
    SEND(_FRAME, push, argument); \
}

#define OP_PUSH_FIELD { \
    uint8_t arg; \
    UNPACK1(pc, arg); \
    pVMSymbol field_name = (pVMSymbol)SEND(method, get_indexable_field, arg); \
    pVMObject self = _SELF; \
    int field_index = SEND(self, get_field_index, field_name); \
    pVMObject o = SEND(self, get_field, field_index); \
    SEND(_FRAME, push, o); \
}

#define OP_PUSH_BLOCK { \
    uint8_t arg; \
    UNPACK1(pc, arg); \
    pVMMethod block_method = (pVMMethod)SEND(method, get_indexable_field, arg); \
    int number_of_arguments = SEND(block_method, get_number_of_arguments); \
    SEND(_FRAME, push, \
         (pVMObject) VMBlock_new_with(block_method, \
                                        _FRAME, \
                                        number_of_arguments)); \
}

#define OP_PUSH_CONSTANT { \
    uint8_t arg; \
    UNPACK1(pc, arg); \
    pVMObject constant = SEND(method, get_indexable_field, arg); \
    SEND(_FRAME, push, constant); \
}

#define OP_PUSH_GLOBAL { \
    static pString uG = NULL; \
    if (!uG) uG = String_new("unknownGlobal:"); \
    uint8_t arg; \
    UNPACK1(pc, arg); \
    pVMSymbol global_name = (pVMSymbol)SEND(method, get_indexable_field, arg); \
    pVMObject global = Universe_get_global(global_name); \
    if(global != NULL) \
        SEND(_FRAME, push, global); \
    else { \
        pVMObject arguments[] = { (pVMObject)global_name }; \
        pVMObject self =_SELF; \
        SEND(self, send, uG, arguments, 1); \
    } \
}

#define OP_POP { \
    SEND(_FRAME, pop); \
} 

#define OP_POP_LOCAL { \
    uint8_t bc1; \
    uint8_t bc2; \
    UNPACK2(pc, bc1, bc2); \
    pVMObject o =  SEND(_FRAME, pop); \
    SEND(_FRAME, set_local, bc1, bc2, o); \
}

#define OP_POP_ARGUMENT { \
    uint8_t bc1; \
    uint8_t bc2; \
    UNPACK2(pc, bc1, bc2); \
    pVMObject o =  SEND(_FRAME, pop); \
    SEND(_FRAME, set_argument, bc1, bc2, o); \
}

#define OP_POP_FIELD { \
    uint8_t arg; \
    UNPACK1(pc, arg); \
    pVMSymbol field_name = (pVMSymbol)SEND(method, get_indexable_field, arg); \
    pVMObject self = _SELF; \
    int field_index = SEND(self, get_field_index, field_name); \
    pVMObject o = SEND(_FRAME, pop); \
    SEND(self, set_field, field_index, o); \
}

#define OP_SEND { \
    uint8_t arg; \
    UNPACK1(pc, arg); \
    pVMSymbol signature = (pVMSymbol)SEND(method, get_indexable_field, arg); \
    int number_of_arguments = Signature_get_number_of_arguments(signature); \
    pVMObject receiver = \
        SEND(_FRAME, get_stack_element, number_of_arguments - 1); \
    send_threaded(signature, SEND(receiver, get_class)); \
}

#define OP_SUPER_SEND { \
    static pString dnu = NULL; \
    if (!dnu) dnu = String_new("doesNotUnderstand:arguments:"); \
    uint8_t arg; \
    UNPACK1(pc, arg); \
    pVMSymbol signature = (pVMSymbol)SEND(method, get_indexable_field, arg); \
    pVMFrame ctxt = SEND(_FRAME, get_outer_context); \
    pVMMethod real_method = SEND(ctxt, get_method); \
    pVMClass holder = TSEND(VMInvokable, real_method, get_holder); \
    pVMClass super = SEND(holder, get_super_class); \
    pVMObject invokable = (pVMObject)SEND(super, lookup_invokable, signature); \
    if(invokable != NULL) \
        TSEND(VMInvokable, invokable, invoke, _FRAME); \
    else { \
        int number_of_arguments = Signature_get_number_of_arguments(signature); \
        pVMObject receiver = SEND(_FRAME, \
                                  get_stack_element, number_of_arguments - 1); \
        pVMArray arguments_array = VMArray_new(number_of_arguments); \
        for(int i = number_of_arguments - 1; i >= 0; i--) { \
            pVMObject o = SEND(_FRAME, pop); \
            SEND(arguments_array, set_indexable_field, i, o); \
        } \
        pVMObject arguments[] = \
        { (pVMObject)signature, (pVMObject)arguments_array }; \
        SEND(receiver, send,  dnu, arguments, 2); \
    } \
}

#define OP_RETURN_LOCAL { \
    pVMObject result = SEND(_FRAME, pop); \
    pop_frame_and_push_result_threaded(result); \
}

#define OP_RETURN_NON_LOCAL { \
    static pString eB = NULL; \
    if (!eB) eB =  String_new("escapedBlock:"); \
    pVMObject result = SEND(_FRAME, pop); \
    pVMFrame context = SEND(_FRAME, get_outer_context); \
    if(!SEND(context, has_previous_frame)) { \
        pVMBlock block = (pVMBlock)SEND(_FRAME, get_argument, 0, 0); \
        pVMFrame prev_frame = SEND(_FRAME, get_previous_frame); \
        pVMFrame outer_context =SEND(prev_frame, get_outer_context); \
        pVMObject sender = SEND(outer_context, get_argument, 0, 0); \
        pVMObject arguments[] = { (pVMObject)block }; \
        pop_frame_threaded(); \
        SEND(sender, send, eB, arguments, 1); \
        return; \
    } \
    while(_FRAME != context) { \
        pop_frame_threaded(); \
    } \
    pop_frame_and_push_result_threaded(result); \
}

/********************
* Definition macros *
********************/

#define OP_DEF(ID, NAME) \
    if (init) { \
        inst_handler_entries[BC_##ID].label = &&NAME; \
        inst_handler_entries[BC_##ID].length = bytecode_lengths[BC_##ID]; \
        inst_handler_entries[BC_##ID].sequence[0] = BC_##ID; \
        inst_handler_entries[BC_##ID].sequence[1] = SUPER_INST_SEQUENCE_SENTINEL; \
    } else { \
        NAME: { \
            PROFILE(BC_##ID);\
            OP_##ID; \
            DISPATCH; \
        } \
    }

// Macro to define a super instruction representing 2 regular instructions
#define OP_DEF2(NAME, ID1, ID2) \
    if (init) { \
        if (interpreter_super_inst_config_mask & (1 << 0)) { \
            inst_handler_entries[super_inst_last_bc].label = &&NAME; \
            inst_handler_entries[super_inst_last_bc].length = bytecode_lengths[BC_##ID1] + bytecode_lengths[BC_##ID2]; \
            inst_handler_entries[super_inst_last_bc].sequence[0] = BC_##ID1; \
            inst_handler_entries[super_inst_last_bc].sequence[1] = BC_##ID2; \
            inst_handler_entries[super_inst_last_bc].sequence[2] = SUPER_INST_SEQUENCE_SENTINEL; \
            super_inst_last_bc++; \
        } \
    } else { \
        NAME: { \
            {OP_##ID1} \
            pc++; \
            {OP_##ID2} \
            DISPATCH; \
        } \
    }

// Macro to define a super instruction representing 3 regular instructions
#define OP_DEF3(NAME, ID1, ID2, ID3) \
    if (init) { \
        if (interpreter_super_inst_config_mask & (1 << 1)) { \
            inst_handler_entries[super_inst_last_bc].label = &&NAME; \
            inst_handler_entries[super_inst_last_bc].length = bytecode_lengths[BC_##ID1] + bytecode_lengths[BC_##ID2] + bytecode_lengths[BC_##ID3]; \
            inst_handler_entries[super_inst_last_bc].sequence[0] = BC_##ID1; \
            inst_handler_entries[super_inst_last_bc].sequence[1] = BC_##ID2; \
            inst_handler_entries[super_inst_last_bc].sequence[2] = BC_##ID3; \
            inst_handler_entries[super_inst_last_bc].sequence[3] = SUPER_INST_SEQUENCE_SENTINEL; \
            super_inst_last_bc++; \
        } \
    } else { \
        NAME: { \
            {OP_##ID1} \
            pc++; \
            {OP_##ID2} \
            pc++; \
            {OP_##ID3} \
            DISPATCH; \
        } \
    }

// Macro to define a super instruction representing 4 regular instructions
#define OP_DEF4(NAME, ID1, ID2, ID3, ID4) \
    if (init) { \
        if (interpreter_super_inst_config_mask & (1 << 2)) { \
            inst_handler_entries[super_inst_last_bc].label = &&NAME; \
            inst_handler_entries[super_inst_last_bc].length = bytecode_lengths[BC_##ID1] + bytecode_lengths[BC_##ID2] + bytecode_lengths[BC_##ID3] + bytecode_lengths[BC_##ID4]; \
            inst_handler_entries[super_inst_last_bc].sequence[0] = BC_##ID1; \
            inst_handler_entries[super_inst_last_bc].sequence[1] = BC_##ID2; \
            inst_handler_entries[super_inst_last_bc].sequence[2] = BC_##ID3; \
            inst_handler_entries[super_inst_last_bc].sequence[3] = BC_##ID4; \
            inst_handler_entries[super_inst_last_bc].sequence[4] = SUPER_INST_SEQUENCE_SENTINEL; \
            super_inst_last_bc++; \
        } \
    } else { \
        NAME: { \
            {OP_##ID1} \
            pc++; \
            {OP_##ID2} \
            pc++; \
            {OP_##ID3} \
            pc++; \
            {OP_##ID4} \
            DISPATCH; \
        } \
    }

inst_handler_entry inst_handler_entries[SUPER_INST_MAX];
int super_inst_first_bc = 0x10;
int super_inst_last_bc = 0x10;

void Interpreter_threaded_start_helper(int init) {
    if (!init) {
        DISPATCH;
    }
    
    OP_DEF(HALT, halt);
    OP_DEF(DUP, dup);
    OP_DEF(PUSH_LOCAL, push_local);
    OP_DEF(PUSH_ARGUMENT, push_argument);
    OP_DEF(PUSH_FIELD, push_field);
    OP_DEF(PUSH_BLOCK, push_block);
    OP_DEF(PUSH_CONSTANT, push_constant);
    OP_DEF(PUSH_GLOBAL, push_global);
    OP_DEF(POP, pop);
    OP_DEF(POP_LOCAL, pop_local);
    OP_DEF(POP_ARGUMENT, pop_argument);
    OP_DEF(POP_FIELD, pop_field);
    OP_DEF(SEND, send);
    OP_DEF(SUPER_SEND, super_send);
    OP_DEF(RETURN_LOCAL, return_local);
    OP_DEF(RETURN_NON_LOCAL, return_non_local);

    // Super instructions of length 4 based on profiling results
    OP_DEF4(op4_1,  POP, PUSH_LOCAL, PUSH_ARGUMENT, SEND);
    OP_DEF4(op4_2,  PUSH_ARGUMENT, PUSH_ARGUMENT, PUSH_CONSTANT, SEND);
    OP_DEF4(op4_3,  PUSH_ARGUMENT, PUSH_ARGUMENT, PUSH_ARGUMENT, SEND);
    OP_DEF4(op4_4,  DUP, POP_LOCAL, POP, PUSH_LOCAL);
    OP_DEF4(op4_5,  POP, PUSH_ARGUMENT, PUSH_ARGUMENT, PUSH_CONSTANT);
    OP_DEF4(op4_6,  DUP, POP_FIELD, POP, PUSH_ARGUMENT);
    OP_DEF4(op4_7,  PUSH_FIELD, PUSH_ARGUMENT, PUSH_ARGUMENT, SEND);
    OP_DEF4(op4_8,  POP_LOCAL, POP, PUSH_LOCAL, SEND);
    
    // Super instructions of length 3 based on profiling results
    OP_DEF3(op3_1,  PUSH_ARGUMENT, PUSH_ARGUMENT, SEND);
    OP_DEF3(op3_2,  POP, PUSH_ARGUMENT, SEND);
    OP_DEF3(op3_3,  PUSH_LOCAL, PUSH_ARGUMENT, SEND);
    OP_DEF3(op3_4,  DUP, POP_LOCAL, RETURN_LOCAL);
    OP_DEF3(op3_5,  PUSH_ARGUMENT, PUSH_LOCAL, SEND);
    OP_DEF3(op3_6,  PUSH_ARGUMENT, PUSH_CONSTANT, SEND);
    OP_DEF3(op3_7,  POP, PUSH_LOCAL, PUSH_ARGUMENT);
    OP_DEF3(op3_8,  DUP, POP_LOCAL, POP);
    
    // Super instructions of length 2 based on profiling results
    OP_DEF2(op2_1,  PUSH_ARGUMENT, SEND);
    OP_DEF2(op2_2,  PUSH_ARGUMENT, PUSH_ARGUMENT);
    OP_DEF2(op2_3,  PUSH_GLOBAL, RETURN_LOCAL);
    OP_DEF2(op2_4,  PUSH_BLOCK, SEND);
    OP_DEF2(op2_5,  POP, PUSH_ARGUMENT);
    OP_DEF2(op2_6,  PUSH_LOCAL, PUSH_ARGUMENT);
    OP_DEF2(op2_7,  DUP, POP_LOCAL);
    OP_DEF2(op2_8,  PUSH_LOCAL, SEND);
    OP_DEF2(op2_9,  PUSH_CONSTANT, SEND);
    OP_DEF2(op2_10, POP, PUSH_LOCAL);
    
    // Adjust super_inst_last_bc to point to the last one and not behind it
    if (init)
        if (interpreter_super_inst_config_mask != 0)
            super_inst_last_bc--;
}

int labelsInitialized = 0;

void Interpreter_threaded_init(void) {
	if (!labelsInitialized)
		Interpreter_threaded_start_helper(1);
	labelsInitialized = 1;
}

void Interpreter_threaded_start(void) {
	if (!labelsInitialized) Interpreter_threaded_init();
	Interpreter_threaded_start_helper(0);
}

pVMFrame Interpreter_threaded_push_new_frame(pVMMethod _method) {
	pVMFrame frame = Interpreter_get_frame();
	if (frame) {
		SEND(frame->bytecode_index, set_embedded_integer, (int)pc); //SEND(frame, set_bytecode_index, (int)pc);
		// use direct access here since the method is adviced
		//TODO: check for problems with int-one this value should be never reached by int-tagging aware code.. hope so 
	}
    method = _method;
    frame = VMFrame_new_with(_FRAME, _method);
    Interpreter_set_frame(frame);
    return frame;
}

void Interpreter_threaded_set_frame(pVMFrame _frame) {
	Interpreter_set_frame(_frame);
    method = SEND(_FRAME, get_method);
    pc = (void**)(SEND(Interpreter_get_frame()->bytecode_index, get_embedded_integer)); //pc = (void**)(SEND(Interpreter_get_frame(), get_bytecode_index));
    // use direct access here since the method is adviced
    //TODO: check for problems with int-one this value should be never reached by int-tagging aware code.. hope so
}

void Interpreter_threaded_set_bytecode_index(int _bytecode_index) {
	if (!method) return;
    pc = pc_from_index(method, _bytecode_index);
}

int Interpreter_threaded_get_bytecode_index() {
    return index_from_pc(method, pc);
}

uint8_t Interpreter_threaded_first_super_bc() {
    return super_inst_first_bc;
}

uint8_t Interpreter_threaded_last_super_bc() {
    return super_inst_last_bc;
}
