#ifndef CORE_SYSTEM_IMAGE_H_
#define CORE_SYSTEM_IMAGE_H_


extern "C" {


/**
 * \brief Hibernates the currently running VM image.
 * 
 * Stores the currently running VM image in a file, so
 * enabling its later reactivation.
 * 
 * The function has one "string argument", the name of
 * the image, from which its filename is derived. This
 * name is also needed during loading.
 * 
 * \note This method is not expected to return. (Though
 * on Smalltalk-level it does - when the image is loaded.)
 */ 
void _System_hibernate_withEncryption_usingKey_(pVMObject object, pVMFrame frame);

/**
 * \brief Inspects an image with the given name.
 * 
 * Returns a Smalltalk object (VMImage on VM side),
 * providing access to meta data of the image.
 */
void _System_inspectImage_(pVMObject object, pVMFrame frame);

/**
 * \brief Restores the image with the given name.
 * 
 * Restores an image from file. The image name (and thus also
 * the file name) is given by the first "string argument".
 * 
 * On Smalltalk level this method is not expected to return.
 * But, on a C level it does, since "only" the Smalltalk object
 * world and interpreter stack has been switched.
 */
void _System_loadImage_usingKey_(pVMObject object, pVMFrame frame);

/**
 * \brief Relocates the heap to a new location.
 * 
 * Moves the heap to a new location to test the relocation
 * functionality.
 */
void _System_rh(pVMObject object, pVMFrame frame);

}

#endif CORE_SYSTEM_IMAGE_H_
