#include <stdint.h>

#include <module.Image>

/**
 * \brief Updates the object references of the key and value of a hashmap entry.
 * 
 * Updates both the object reference to the key and the value of a given
 * hasmap entry. This, of course, only if these are GC-managed objects.
 * 
 * \param _self The entry from which to update key and value.
 * \param difference The heap-address difference used to update the pointers.
 */ 
void  _HashmapElem_update_references(void* _self, const intptr_t difference) {
	pHashmapElem self = (pHashmapElem) _self;
	
	if (gc_is_managed_object((pVMObject)((intptr_t)self->key + difference))) {
		self->key = (pVMObject)((intptr_t)self->key + difference);
		SEND((pVMObject)self->key, update_references, difference);
	}
	
	if (gc_is_managed_object((pVMObject)((intptr_t)self->value + difference))) {
		self->value = (pVMObject)((intptr_t)self->value + difference);
		SEND((pVMObject)self->value, update_references, difference);
	}	
}

/**
 * \brief Updates the references to heap objects in the map.
 * 
 * In case of a relocated heap, either after image loading or otherwise,
 * the pointers to gc-managed objects must be adjusted. This method
 * performs this task for the hashmap (and its contained object by
 * recursively calling the method on the objects themselves).
 * 
 * \param _self The hashmap self pointer.
 * \param difference The difference of the two heaps in respect to their
 * 		starting addresses. The difference must be added to a heap object
 *      in order to update the pointer for the new heap.
 */ 
void _Hashmap_update_references(void* _self, const intptr_t difference) {
    pHashmap self = (pHashmap)_self;
    
    for(size_t i = 0; i < self->size; ++i) {
    	if (self->elems[i]) {
    		if (IS_A(self->elems[i], VMClass)) {
    			pVMClass c = (pVMClass)self->elems[i];
    		}
    		SEND(self->elems[i], update_references, difference);
    	}
    }
}
