/* Native multi-threading for CSOM - Mutex primitive class
 * 
 * Copyright (C) 2007 by Martin Beck and Jan Klimke
 * 
 * created 2007-05-09
 */
 
#include <module.NativeThreads>
#include <module.Memory>
#include <module.VM>

#include <stdio.h>

pMutexId _VMMutex_get_embedded_mutex_id(pVMMutex);
void     _VMMutex_lock(pVMMutex);
void     _VMMutex_unlock(pVMMutex);
bool     _VMMutex_is_locked(pVMMutex);

void _VMMutex_init(void* _self, ...);
void _VMMutex_free(void *self);

pVMMutex VMMutex_new() {
	pVMMutex result = (pVMMutex) memory_allocate(sizeof(VMMutex));
    if(result) {
        result->_vtable = VMMutex_vtable();
        INIT(result);
    }
    return result;
}

void _VMMutex_init(void* _self, void* param1, void* param2) {
	pVMMutex self = (pVMMutex)_self;
	
	size_t size = NUMBER_OF_OBJECT_FIELDS(VMObject);
	SUPER(VMObject, self, init, &size, NULL);
    
	// create a new pthread mutex and embedd it in the CSOM mutex
	int err = pthread_mutex_init(&(((pVMMutex)self)->embedded_mutex_id), NULL);
	if(err != 0){
		fprintf(stderr, "Could not initialise mutex.\n");
	}
	
	SEND(self, set_class, mutex_class);
}

void _VMMutex_free(void *self) {
    // to be done later, somehow
}

pMutexId _VMMutex_get_embedded_mutex_id(void* _self) {
	pVMMutex mutex_object = (pVMMutex)_self;
	return &mutex_object->embedded_mutex_id;
}

void _VMMutex_set_embedded_mutex_id(void* _self, MutexId value) {
	pVMMutex mutex_object = (pVMMutex)_self;
	mutex_object->embedded_mutex_id = value;
}


void _VMMutex_lock(void* _self) {
	pVMMutex self = (pVMMutex)_self;
	pthread_mutex_lock(&self->embedded_mutex_id);
}


void _VMMutex_unlock(void* _self) {
	pVMMutex self = (pVMMutex)_self;
	pthread_mutex_unlock(&self->embedded_mutex_id);
}

bool _VMMutex_is_locked(void* _self) {
	pVMMutex self = (pVMMutex)_self;
	int res = pthread_mutex_trylock(&self->embedded_mutex_id);
	 
	if( res == 0){
		pthread_mutex_unlock(&self->embedded_mutex_id);
		return true;	
	}else{		
		return false;
	}
}
