/* Native multi-threading for CSOM - Thread primitive class
 * 
 * Copyright (C) 2007 by Martin Beck and Jan Klimke
 * 
 * created 2007-05-09
 */
 
#include <module.NativeThreads>
#include <module.Memory>
#include <module.VM>

#include <sched.h>

/*ThreadId (*get_embedded_thread_id)(pVMThread);
void     (*set_embedded_thread_id)(pVMThread, ThreadId);
void     (*join)(pVMThread, int*);*/

pVMThread VMThread_new() {
	pVMThread result = (pVMThread) memory_allocate(sizeof(VMThread));
    if(result) {
        result->_vtable = VMThread_vtable();
        size_t size = NUMBER_OF_OBJECT_FIELDS(VMThread) - 1; 
        INIT1(result, &size);
    }
    SEND(result, set_class, thread_class);
    return result;
}


pVMSignal _VMThread_get_resume_signal(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->resume_signal;
}


void _VMThread_set_resume_signal(void* _self, pVMSignal value) {
    pVMThread self = (pVMThread)_self;
    
    self->resume_signal = value;
}


bool _VMThread_should_stop(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->should_stop == true_object;
}


void _VMThread_set_should_stop(void* _self, bool value) {
    pVMThread self = (pVMThread)_self;
    
    if (value) {
    	self->should_stop = true_object;
    } else {
    	self->should_stop = false_object;
    }
}

pVMBlock _VMThread_get_block_to_run(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->block_to_run;
}


void _VMThread_set_block_to_run(void* _self, pVMBlock value) {
    pVMThread self = (pVMThread)_self;
    
    self->block_to_run = value;
}


pVMObject _VMThread_get_argument(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->argument;
}


void _VMThread_set_argument(void* _self, pVMObject value) {
    pVMThread self = (pVMThread)_self;
    
    self->argument = value;
}


pVMString _VMThread_get_name(void* _self) {
    pVMThread self = (pVMThread)_self;
    
    return self->name;
}


void _VMThread_set_name(void* _self, pVMString value) {
    pVMThread self = (pVMThread)_self;
    
    self->name = value;
}


ThreadId _VMThread_get_embedded_thread_id(void* _self) {
	pVMThread self = (pVMThread)_self;
	return self->embedded_thread_id;
}

void _VMThread_set_embedded_thread_id(void* _self, ThreadId value) {
	pVMThread self = (pVMThread)_self;
	self->embedded_thread_id = value;
}

void VMThread_create_safe_global(ThreadSafeGlobal *global) {
	pthread_key_create(global, NULL);
}

void VMThread_delete_safe_global(ThreadSafeGlobal global) {
	pthread_key_delete(global);
}

void* VMThread_get_safe_global(ThreadSafeGlobal global) {
	return pthread_getspecific(global);
}

void VMThread_set_safe_global(ThreadSafeGlobal global, void* value) {
	pthread_setspecific(global, value);
}

void _VMThread_join(void* _self, int *exit_status) {
	pVMThread self = (pVMThread)_self;
	pthread_join(self->embedded_thread_id, (void **)exit_status);
}

void VMThread_yield() {
	sched_yield();
}
