#ifndef TAGGED_INT_ONE_H_
#define TAGGED_INT_ONE_H_

#include <stdint.h>
#include <stdio.h>

// SMELL: TODO: fix smell
typedef struct _VMClass VMClass, *pVMClass;
typedef struct _VMInteger VMInteger, *pVMInteger;
typedef struct _VMObject VMObject, *pVMObject;


/**
 * max value for tagged integers
 * 01111111 11111111 11111111 1111111X  
 */
const int32_t VMTAGGEDINTEGER_MAX = 1073741823;

/**
 * min value for tagged integers
 * 10000000 00000000 00000000 0000000X  
 */
const int32_t VMTAGGEDINTEGER_MIN = -1073741824;


/**
 * set an tagged integer
 */
#define VMINTEGER_SET(O,V) \
    ({ \
	 if (VMINTEGER_IS_TAGGED(O)) { O = (typeof(O))VMINTEGER_TAG(V); } \
     else { SEND((pVMInteger)O, set_embedded_integer,V); } \
    })


/**
 * helper to convert an integer value into a tagged integer value
 */
#if DEBUG
	#define VMINTEGER_TAG(v) \
	({ intptr_t value = (intptr_t)v; \
		if (value < VMTAGGEDINTEGER_MIN || value > VMTAGGEDINTEGER_MAX) { \
			printf("Failed to tag integer %p. Out of range. In file %s on line %d Min: %d Max: %d, %p, %p\n", value, __FILE__, __LINE__, VMTAGGEDINTEGER_MIN, VMTAGGEDINTEGER_MAX, value < VMTAGGEDINTEGER_MIN, value > VMTAGGEDINTEGER_MAX);  \
			Universe_exit(ERR_PANIC);									 \
	    } \
		_VMINTEGER_TAG(value); \
	})
#else
	#define VMINTEGER_TAG(value) _VMINTEGER_TAG(value)
#endif
	
#define _VMINTEGER_TAG(value) ((void*)((value << 1) | VMINTEGER_TAG_BIT))


/**
 * helper to convert a tagged integer value into a untagged integer value
 */
#if DEBUG
	#define VMINTEGER_UNTAG(tagged) \
	({ \
		if(!VMINTEGER_IS_TAGGED(tagged)) { \
			printf("Given value is not a tagged int %d.\n", (int32_t)tagged); \
			Universe_exit(ERR_PANIC);							 \
	    } \
		_VMINTEGER_UNTAG(tagged); \
	})
#else
	#define VMINTEGER_UNTAG(tagged) _VMINTEGER_UNTAG(tagged)
#endif

#define _VMINTEGER_UNTAG(tagged) ((int32_t)tagged >> 1)

// TODO: generated this funtion definitions and get ridd of them entierly in hand written code
// overwritten parent methods
// forward definitions are used in VMTable construction, this code is still not modularized, look at VMInteger.c
void _VMInteger_free(void* _self);
int32_t _VMInteger_object_size(void* _self);
pVMClass _VMInteger_get_class(void* _self);
void _VMInteger_set_class(void* _self, pVMClass value);
pVMObject _VMInteger_get_field(void* _self, int index);
void _VMInteger_set_field(void* _self, int index, pVMObject value);
int _VMInteger_get_number_of_fields(void* _self);


#endif TAGGED_INT_ONE_H_
